import matplotlib
import numpy as np
import pytest

import act
from act.plotting import GeographicPlotDisplay
from act.tests import sample_files

try:
    import cartopy  # noqa

    CARTOPY_AVAILABLE = True
except ImportError:
    CARTOPY_AVAILABLE = False

matplotlib.use('Agg')


@pytest.mark.skipif(not CARTOPY_AVAILABLE, reason='Cartopy is not installed.')
@pytest.mark.mpl_image_compare(style='default', tolerance=10)
def test_geoplot():
    sonde_ds = act.io.arm.read_arm_netcdf(sample_files.EXAMPLE_SONDE1)
    geodisplay = GeographicPlotDisplay({'sgpsondewnpnC1.b1': sonde_ds}, figsize=(15, 8))
    try:
        geodisplay.geoplot(
            'tdry',
            cartopy_feature=[
                'STATES',
                'LAND',
                'OCEAN',
                'COASTLINE',
                'BORDERS',
                'LAKES',
                'RIVERS',
            ],
            text={'Ponca City': [-97.0725, 36.7125]},
            img_tile=None,
        )
        try:
            return geodisplay.fig
        finally:
            matplotlib.pyplot.close(geodisplay.fig)
    except Exception:
        pass
    sonde_ds.close()


@pytest.mark.skipif(not CARTOPY_AVAILABLE, reason='Cartopy is not installed.')
@pytest.mark.mpl_image_compare(style='default', tolerance=10)
def test_geoplot_tile():
    sonde_ds = act.io.arm.read_arm_netcdf(sample_files.EXAMPLE_SONDE1)
    geodisplay = GeographicPlotDisplay({'sgpsondewnpnC1.b1': sonde_ds}, figsize=(15, 8))
    try:
        geodisplay.geoplot(
            'tdry',
            cartopy_feature=[
                'STATES',
                'LAND',
                'OCEAN',
                'COASTLINE',
                'BORDERS',
                'LAKES',
                'RIVERS',
            ],
            text={'Ponca City': [-97.0725, 36.7125]},
            img_tile='GoogleTiles',
            img_tile_args={'style': 'terrain'},
        )
        try:
            return geodisplay.fig
        finally:
            matplotlib.pyplot.close(geodisplay.fig)
    except Exception:
        pass
    sonde_ds.close()


def test_geo_errors():
    sonde_ds = act.io.arm.read_arm_netcdf(sample_files.EXAMPLE_SONDE1)
    display = GeographicPlotDisplay({'thing1': sonde_ds, 'thing2': sonde_ds})
    with np.testing.assert_raises(ValueError):
        display.geoplot('tdry')
    with np.testing.assert_raises(ValueError):
        display.geoplot()

    del sonde_ds['lat']
    display = GeographicPlotDisplay({'thing1': sonde_ds, 'thing2': sonde_ds})
    with np.testing.assert_raises(ValueError):
        display.geoplot('tdry')

    sonde_ds = act.io.arm.read_arm_netcdf(sample_files.EXAMPLE_SONDE1)
    del sonde_ds['lon']
    del sonde_ds['tdry'].attrs['units']
    display = GeographicPlotDisplay({'thing1': sonde_ds, 'thing2': sonde_ds})
    with np.testing.assert_raises(ValueError):
        display.geoplot('tdry')



import act
import glob
import numpy as np
from numpy.testing import assert_equal
import xarray as xr
import pytest


def test_read_ameriflux():
    rename_vars_dict = {'FETCH_70': 'foo'}

    ds = act.io.read_ameriflux(
        act.tests.EXAMPLE_AMERIFLUX_BASE,
        metadata_filename=act.tests.EXAMPLE_AMERIFLUX_META,
        data_type='BASE',
        rename_vars_dict=rename_vars_dict,
    )
    # Test if variables are correct and units and fill values were added
    assert 'H2O' in ds.variables.keys()
    assert ds.variables['H2O'].attrs['units'] == 'mmolH2O mol-1'
    assert ds.variables['H2O'].attrs['_FILL_VALUE'] == -9999
    assert np.isnan(ds.variables['H2O'].values[0:5]).all()
    assert_equal(ds.variables['H2O'].values[-5:], [6.40095, 6.31573, 6.04937, 6.04294, 5.70889])

    assert 'T_SONIC' in ds.variables.keys()
    assert ds.variables['T_SONIC'].attrs['units'] == 'deg C'
    assert ds.variables['T_SONIC'].attrs['_FILL_VALUE'] == -9999
    assert np.isnan(ds.variables['T_SONIC'].values[0:5]).all()
    assert_equal(ds.variables['T_SONIC'].values[-5:], [4.30861, 4.19665, 4.16113, 3.99634, 3.86785])

    # Test if time is correct
    assert ds.time.values[0] == np.datetime64('2024-01-01T00:00:00.000000000')
    assert len(ds.time.values) == 17568

    # Test if renaming variables worked
    assert 'foo' in ds.variables.keys()

    # Test metadata
    assert ds.attrs['site'] == 'US-CU1'
    assert ds.attrs['version'] == '1-5'
    assert ds.attrs['SITE_NAME'] == 'UIC Plant Research Laboratory Chicago'

    assert len(ds.attrs['TEAM_MEMBERS']) == 9
    assert 'Bhupendra' in ds.attrs['TEAM_MEMBERS'][0]
    assert 'Matt' in ds.attrs['TEAM_MEMBERS'][6]
    assert 'TEAM_MEMBER_EMAIL' in ds.attrs['TEAM_MEMBERS'][0]
    assert 'TEAM_MEMBER_INSTITUTION' in ds.attrs['TEAM_MEMBERS'][0]
    assert 'TEAM_MEMBER_INSTITUTION:Argonne National Laboratory' in ds.attrs['TEAM_MEMBERS'][0]

    assert 'FLUX_MEASUREMENTS_VARIABLE:CO2' in ds.attrs['FLUX_MEASUREMENTS_METHODS'][0]
    assert 'FLUX_MEASUREMENTS_VARIABLE:H' in ds.attrs['FLUX_MEASUREMENTS_METHODS'][1]
    assert 'FLUX_MEASUREMENTS_METHOD:Eddy Covariance' in ds.attrs['FLUX_MEASUREMENTS_METHODS'][0]

    assert 'Bhupendra' in ds.attrs['DOI_CONTRIBUTORS'][0]
    assert 'Sujan' in ds.attrs['DOI_CONTRIBUTORS'][1]
    assert 'DOI_CONTRIBUTOR_DATAPRODUCT:AmeriFlux' in ds.attrs['DOI_CONTRIBUTORS'][0]
    assert 'DOI_CONTRIBUTOR_ROLE:Author' in ds.attrs['DOI_CONTRIBUTORS'][0]


def test_convert_to_ameriflux():
    files = glob.glob(act.tests.sample_files.EXAMPLE_ECORSF_E39)
    ds_ecor = act.io.arm.read_arm_netcdf(files)

    with pytest.warns(UserWarning, match="mapping was not provided"):
        df = act.io.ameriflux.convert_to_ameriflux(ds_ecor)

    assert 'FC' in df
    assert 'WS_MAX' in df

    files = glob.glob(act.tests.sample_files.EXAMPLE_SEBS_E39)
    ds_sebs = act.io.arm.read_arm_netcdf(files)

    ds = xr.merge([ds_ecor, ds_sebs])
    with pytest.warns(UserWarning, match="mapping was not provided"):
        df = act.io.ameriflux.convert_to_ameriflux(ds)

    assert 'SWC_2_1_1' in df
    assert 'TS_3_1_1' in df
    assert 'G_2_1_1' in df

    files = glob.glob(act.tests.sample_files.EXAMPLE_STAMP_E39)
    ds_stamp = act.io.arm.read_arm_netcdf(files)

    ds = xr.merge([ds_ecor, ds_sebs, ds_stamp], compat='override')
    with pytest.warns(UserWarning, match="mapping was not provided"):
        df = act.io.ameriflux.convert_to_ameriflux(ds)

    assert 'SWC_6_10_1' in df
    assert 'G_2_1_1' in df
    assert 'TS_5_2_1' in df



"""
Stores the class for SkewTDisplay.

"""

from copy import deepcopy

import matplotlib.pyplot as plt

# Import third party libraries
import metpy
import metpy.calc as mpcalc
import numpy as np
import scipy
from metpy.plots import Hodograph, SkewT
from metpy.units import units

from ..retrievals import calculate_stability_indicies

# Import Local Libs
from ..utils import datetime_utils as dt_utils
from .plot import Display


class SkewTDisplay(Display):
    """
    A class for making Skew-T plots.

    This is inherited from the :func:`act.plotting.Display`
    class and has therefore has the same attributes as that class.
    See :func:`act.plotting.Display`
    for more information. There are no additional attributes or parameters
    to this class.

    In order to create Skew-T plots, ACT needs the MetPy package to be
    installed on your system. More information about
    MetPy go here: https://unidata.github.io/MetPy/latest/index.html.

    Examples
    --------
    Here is an example of how to make a Skew-T plot using ACT:

    .. code-block :: python

        sonde_ds = act.io.arm.read_arm_netcdf(
           act.tests.sample_files.EXAMPLE_SONDE1)

        skewt = act.plotting.SkewTDisplay(sonde_ds)
        skewt.plot_from_u_and_v('u_wind', 'v_wind', 'pres', 'tdry', 'dp')
        plt.show()

    """

    def __init__(self, ds, subplot_shape=(1,), subplot=None, ds_name=None, set_fig=None, **kwargs):
        # We want to use our routine to handle subplot adding, not the main
        # one
        new_kwargs = kwargs.copy()
        super().__init__(ds, None, ds_name, subplot_kw=dict(projection='skewx'), **new_kwargs)

        # Make a SkewT object for each subplot
        self.add_subplots(subplot_shape, set_fig=set_fig, subplot=subplot, **kwargs)

    def add_subplots(self, subplot_shape=(1,), set_fig=None, subplot=None, **kwargs):
        """
        Adds subplots to the Display object. The current
        figure in the object will be deleted and overwritten.

        Parameters
        ----------
        subplot_shape : 1 or 2D tuple, list, or array
            The structure of the subplots in (rows, cols).
        subplot_kw : dict, optional
            The kwargs to pass into fig.subplots.
        set_fig : matplotlib figure, optional
            Figure to pass to SkewT
        **kwargs : keyword arguments
            Any other keyword arguments that will be passed
            into :func:`matplotlib.pyplot.figure` when the figure
            is made. The figure is only made if the *fig*
            property is None. See the matplotlib
            documentation for further details on what keyword
            arguments are available.

        """
        del self.axes
        if self.fig is None and set_fig is None:
            self.fig = plt.figure(**kwargs)
        if set_fig is not None:
            self.fig = set_fig
        self.SkewT = np.empty(shape=subplot_shape, dtype=SkewT)
        self.axes = np.empty(shape=subplot_shape, dtype=plt.Axes)
        if len(subplot_shape) == 1:
            for i in range(subplot_shape[0]):
                if subplot is None:
                    subplot_tuple = (subplot_shape[0], 1, i + 1)
                else:
                    subplot_tuple = subplot
                self.SkewT[i] = SkewT(fig=self.fig, subplot=subplot_tuple)
                self.axes[i] = self.SkewT[i].ax
        elif len(subplot_shape) == 2:
            for i in range(subplot_shape[0]):
                for j in range(subplot_shape[1]):
                    subplot_tuple = (
                        subplot_shape[0],
                        subplot_shape[1],
                        i * subplot_shape[1] + j + 1,
                    )
                    self.SkewT[i, j] = SkewT(fig=self.fig, subplot=subplot_tuple)
                    self.axes[i, j] = self.SkewT[i, j].ax
        else:
            raise ValueError('Subplot shape must be 1 or 2D!')

    def set_xrng(self, xrng, subplot_index=(0,)):
        """
        Sets the x range of the plot.

        Parameters
        ----------
        xrng : 2 number array.
            The x limits of the plot.
        subplot_index : 1 or 2D tuple, list, or array
            The index of the subplot to set the x range of.

        """
        if self.axes is None:
            raise RuntimeError('set_xrng requires the plot to be displayed.')

        if not hasattr(self, 'xrng') or np.all(self.xrng == 0):
            if len(self.axes.shape) == 2:
                self.xrng = np.zeros((self.axes.shape[0], self.axes.shape[1], 2))
            else:
                self.xrng = np.zeros((self.axes.shape[0], 2))

        self.axes[subplot_index].set_xlim(xrng)
        self.xrng[subplot_index, :] = np.array(xrng)

    def set_yrng(self, yrng, subplot_index=(0,)):
        """
        Sets the y range of the plot.

        Parameters
        ----------
        yrng : 2 number array
            The y limits of the plot.
        subplot_index : 1 or 2D tuple, list, or array
            The index of the subplot to set the x range of.

        """
        if self.axes is None:
            raise RuntimeError('set_yrng requires the plot to be displayed.')

        if not hasattr(self, 'yrng') or np.all(self.yrng == 0):
            if len(self.axes.shape) == 2:
                self.yrng = np.zeros((self.axes.shape[0], self.axes.shape[1], 2))
            else:
                self.yrng = np.zeros((self.axes.shape[0], 2))

        if not hasattr(self, 'yrng') and len(self.axes.shape) == 2:
            self.yrng = np.zeros((self.axes.shape[0], self.axes.shape[1], 2))
        elif not hasattr(self, 'yrng') and len(self.axes.shape) == 1:
            self.yrng = np.zeros((self.axes.shape[0], 2))

        if yrng[0] == yrng[1]:
            yrng[1] = yrng[1] + 1

        self.axes[subplot_index].set_ylim(yrng)
        self.yrng[subplot_index, :] = yrng

    def plot_from_spd_and_dir(
        self, spd_field, dir_field, p_field, t_field, td_field, dsname=None, **kwargs
    ):
        """
        This plot will make a sounding plot from wind data that is given
        in speed and direction.

        Parameters
        ----------
        spd_field : str
            The name of the field corresponding to the wind speed.
        dir_field : str
            The name of the field corresponding to the wind direction
            in degrees from North.
        p_field : str
            The name of the field containing the atmospheric pressure.
        t_field : str
            The name of the field containing the atmospheric temperature.
        td_field : str
            The name of the field containing the dewpoint.
        dsname : str or None
            The name of the datastream to plot. Set to None to make ACT
            attempt to automatically determine this.
        kwargs : dict
            Additional keyword arguments will be passed into
            :func:`act.plotting.SkewTDisplay.plot_from_u_and_v`

        Returns
        -------
        ax : matplotlib axis handle
            The matplotlib axis handle corresponding to the plot.

        """
        if dsname is None and len(self._ds.keys()) > 1:
            raise ValueError(
                'You must choose a datastream when there are 2 '
                'or more datasets in the TimeSeriesDisplay '
                'object.'
            )
        elif dsname is None:
            dsname = list(self._ds.keys())[0]

        # Make temporary field called tempu, tempv
        spd = self._ds[dsname][spd_field].values * units(self._ds[dsname][spd_field].attrs['units'])
        dir = self._ds[dsname][dir_field].values * units(self._ds[dsname][dir_field].attrs['units'])
        tempu, tempv = mpcalc.wind_components(spd, dir)

        self._ds[dsname]['temp_u'] = deepcopy(self._ds[dsname][spd_field])
        self._ds[dsname]['temp_v'] = deepcopy(self._ds[dsname][spd_field])
        self._ds[dsname]['temp_u'].values = tempu
        self._ds[dsname]['temp_v'].values = tempv
        the_ax = self.plot_from_u_and_v(
            'temp_u', 'temp_v', p_field, t_field, td_field, dsname, **kwargs
        )
        del self._ds[dsname]['temp_u'], self._ds[dsname]['temp_v']
        return the_ax

    def plot_from_u_and_v(
        self,
        u_field,
        v_field,
        p_field,
        t_field,
        td_field,
        dsname=None,
        subplot_index=(0,),
        p_levels_to_plot=None,
        show_parcel=True,
        shade_cape=True,
        shade_cin=True,
        set_title=None,
        smooth_p=3,
        plot_dry_adiabats=False,
        plot_moist_adiabats=False,
        plot_mixing_lines=False,
        plot_barbs_kwargs=dict(),
        plot_kwargs=dict(),
        dry_adiabats_kwargs=dict(),
        moist_adiabats_kwargs=dict(),
        mixing_lines_kwargs=dict(),
    ):
        """
        This function will plot a Skew-T from a sounding dataset. The wind
        data must be given in u and v.

        Parameters
        ----------
        u_field : str
            The name of the field containing the u component of the wind.
        v_field : str
            The name of the field containing the v component of the wind.
        p_field : str
            The name of the field containing the pressure.
        t_field : str
            The name of the field containing the temperature.
        td_field : str
            The name of the field containing the dewpoint temperature.
        dsname : str or None
            The name of the datastream to plot. Set to None to make ACT
            attempt to automatically determine this.
        subplot_index : tuple
            The index of the subplot to make the plot on.
        p_levels_to_plot : 1D array
            The pressure levels to plot the wind barbs on. Set to None
            to have ACT to use neatly spaced defaults of
            25, 50, 75, 100, 150, 200, 250, 300, 400, 500, 600, 700, 750, 800,
            850, 900, 950, and 1000 hPa.
        show_parcel : bool
            Set to true to calculate the profile a parcel takes through the atmosphere
            using the metpy.calc.parcel_profile function.  From their documentation,
            the parcel starts at the surface temperature and dewpoint, is lifted up
            dry adiabatically to the LCL and then moist adiabatically from there.
        shade_cape : bool
            Set to True to shade the CAPE red.
        shade_cin : bool
            Set to True to shade the CIN blue.
        set_title : None or str
            The title of the plot is set to this. Set to None to use
            a default title.
        smooth_p : int
            If pressure is not in descending order, will smooth the data
            using this many points to try and work around the issue.
            Default is 3 but inthe pbl retrieval code we have to default to 5 at times
        plot_barbs_kwargs : dict
            Additional keyword arguments to pass into MetPy's
            SkewT.plot_barbs.
        plot_kwargs : dict
            Additional keyword arguments to pass into MetPy's
            SkewT.plot.
        dry_adiabats_kwargs : dict
            Additional keyword arguments to pass into MetPy's plot_dry_adiabats function
        moist_adiabats_kwargs : dict
            Additional keyword arguments to pass into MetPy's plot_moist_adiabats function
        mixing_lines_kwargs : dict
            Additional keyword arguments to pass into MetPy's plot_mixing_lines function

        Returns
        -------
        ax : matplotlib axis handle
            The axis handle to the plot.

        References
        ----------
        May, R. M., Arms, S. C., Marsh, P., Bruning, E., Leeman, J. R., Goebbert, K., Thielen, J. E.,
            Bruick, Z., and Camron, M. D., 2023: MetPy: A Python Package for Meteorological Data.
            Unidata, Unidata/MetPy, doi:10.5065/D6WW7G29.

        """
        if dsname is None and len(self._ds.keys()) > 1:
            raise ValueError(
                'You must choose a datastream when there are 2 '
                'or more datasets in the TimeSeriesDisplay '
                'object.'
            )
        elif dsname is None:
            dsname = list(self._ds.keys())[0]

        if p_levels_to_plot is None:
            p_levels_to_plot = np.array(
                [
                    25.0,
                    50.0,
                    75.0,
                    100.0,
                    150.0,
                    200.0,
                    250.0,
                    300.0,
                    400.0,
                    500.0,
                    600.0,
                    700.0,
                    750.0,
                    800.0,
                    850.0,
                    900.0,
                    950.0,
                    1000.0,
                ]
            ) * units('hPa')

        # Get pressure and smooth if not in order
        p = self._ds[dsname][p_field]
        if not all(p[i] <= p[i + 1] for i in range(len(p) - 1)):
            if 'time' in self._ds:
                self._ds[dsname][p_field] = (
                    self._ds[dsname][p_field]
                    .rolling(time=smooth_p, min_periods=1, center=True)
                    .mean()
                )
                p = self._ds[dsname][p_field]

        p_units = self._ds[dsname][p_field].attrs['units']
        p = p.values * getattr(units, p_units)
        if len(np.shape(p)) == 2:
            p = np.reshape(p, p.shape[0] * p.shape[1])

        T = self._ds[dsname][t_field]
        T_units = self._ds[dsname][t_field].attrs['units']
        if T_units == 'C':
            T_units = 'degC'

        T = T.values * getattr(units, T_units)
        if len(np.shape(T)) == 2:
            T = np.reshape(T, T.shape[0] * T.shape[1])

        Td = self._ds[dsname][td_field]
        Td_units = self._ds[dsname][td_field].attrs['units']
        if Td_units == 'C':
            Td_units = 'degC'

        Td = Td.values * getattr(units, Td_units)
        if len(np.shape(Td)) == 2:
            Td = np.reshape(Td, Td.shape[0] * Td.shape[1])

        u = self._ds[dsname][u_field]
        u_units = self._ds[dsname][u_field].attrs['units']
        u = u.values * getattr(units, u_units)
        if len(np.shape(u)) == 2:
            u = np.reshape(u, u.shape[0] * u.shape[1])

        v = self._ds[dsname][v_field]
        v_units = self._ds[dsname][v_field].attrs['units']
        v = v.values * getattr(units, v_units)
        if len(np.shape(v)) == 2:
            v = np.reshape(v, v.shape[0] * v.shape[1])

        u_red = np.zeros_like(p_levels_to_plot) * getattr(units, u_units)
        v_red = np.zeros_like(p_levels_to_plot) * getattr(units, v_units)

        # Check p_levels_to_plot units, and convert to p units if needed
        if not hasattr(p_levels_to_plot, 'units'):
            p_levels_to_plot = p_levels_to_plot * getattr(units, p_units)
        else:
            p_levels_to_plot = p_levels_to_plot.to(p_units)

        for i in range(len(p_levels_to_plot)):
            index = np.argmin(np.abs(p_levels_to_plot[i] - p))
            u_red[i] = u[index].magnitude * getattr(units, u_units)
            v_red[i] = v[index].magnitude * getattr(units, v_units)

        self.SkewT[subplot_index].plot(p, T, 'r', **plot_kwargs)
        self.SkewT[subplot_index].plot(p, Td, 'g', **plot_kwargs)
        self.SkewT[subplot_index].plot_barbs(
            p_levels_to_plot.magnitude, u_red, v_red, **plot_barbs_kwargs
        )

        # Metpy fix if Pressure does not decrease monotonically in
        # your sounding.
        try:
            prof = mpcalc.parcel_profile(p, T[0], Td[0]).to('degC')
        except metpy.calc.exceptions.InvalidSoundingError:
            p = scipy.ndimage.median_filter(p, 3, output=float)
            p = metpy.units.units.Quantity(p, p_units)
            prof = mpcalc.parcel_profile(p, T[0], Td[0]).to('degC')

        if show_parcel:
            # Only plot where prof > T
            lcl_pressure, lcl_temperature = mpcalc.lcl(p[0], T[0], Td[0])
            self.SkewT[subplot_index].plot(
                lcl_pressure, lcl_temperature, 'ko', markerfacecolor='black', **plot_kwargs
            )
            self.SkewT[subplot_index].plot(p, prof, 'k', linewidth=2, **plot_kwargs)

        if shade_cape:
            self.SkewT[subplot_index].shade_cape(p, T, prof, linewidth=2)

        if shade_cin:
            self.SkewT[subplot_index].shade_cin(p, T, prof, linewidth=2)

        # Get plot temperatures from x-axis as t0
        t0 = self.SkewT[subplot_index].ax.get_xticks() * getattr(units, T_units)

        # Add minimum pressure to pressure levels to plot
        if np.nanmin(p.magnitude) < np.nanmin(p_levels_to_plot.magnitude):
            plp = np.insert(p_levels_to_plot.magnitude, 0, np.nanmin(p.magnitude)) * units('hPa')
        else:
            plp = p_levels_to_plot

        # New options for plotting dry and moist adiabats as well as the mixing lines
        if plot_dry_adiabats:
            self.SkewT[subplot_index].plot_dry_adiabats(pressure=plp, t0=t0, **dry_adiabats_kwargs)

        if plot_moist_adiabats:
            self.SkewT[subplot_index].plot_moist_adiabats(
                t0=t0, pressure=plp, **moist_adiabats_kwargs
            )

        if plot_mixing_lines:
            self.SkewT[subplot_index].plot_mixing_lines(pressure=plp, **mixing_lines_kwargs)

        # Set Title
        if set_title is None:
            if 'time' in self._ds[dsname]:
                title_time = (dt_utils.numpy_to_arm_date(self._ds[dsname].time.values[0]),)
            elif '_file_dates' in self._ds[dsname].attrs:
                title_time = self._ds[dsname].attrs['_file_dates'][0]
            else:
                title_time = ''
            set_title = ' '.join([dsname, 'on', title_time[0]])

        self.axes[subplot_index].set_title(set_title)

        # Set Y Limit
        our_data = p.magnitude
        if np.isfinite(our_data).any():
            yrng = [np.nanmax(our_data), np.nanmin(our_data)]
        else:
            yrng = [1000.0, 100.0]
        self.set_yrng(yrng, subplot_index)

        # Set X Limit
        xrng = [np.nanmin(T.magnitude) - 10.0, np.nanmax(T.magnitude) + 10.0]
        self.set_xrng(xrng, subplot_index)

        return self.axes[subplot_index]

    def plot_hodograph(
        self,
        spd_field,
        dir_field,
        color_field=None,
        set_fig=None,
        set_axes=None,
        component_range=80,
        dsname=None,
        uv_flag=False,
    ):
        """
        This will plot a hodograph from the radiosonde wind data using
        MetPy

        Parameters
        ----------
        spd_field : str
            The name of the field corresponding to the wind speed.
        dir_field : str
            The name of the field corresponding to the wind direction
            in degrees from North.
        color_field : str, optional
            The name of the field if wanting to shade by another variable
        set_fig : matplotlib figure, optional
            The figure to plot on
        set_axes : matplotlib axes, optional
            The specific axes to plot on
        component_range : int
             Range of the hodograph.  Default is 80
        dsname : str
             Name of the datastream to plot if multiple in the plot object
        uv_flag : boolean
             If set to True, spd_field and dir_field will be treated as the
             U and V wind variable names

        Returns
        -------
        self.axes : matplotlib axes

        """

        if dsname is None and len(self._ds.keys()) > 1:
            raise ValueError(
                'You must choose a datastream when there are 2 '
                'or more datasets in the TimeSeriesDisplay '
                'object.'
            )
        elif dsname is None:
            dsname = list(self._ds.keys())[0]

        # Get the current plotting axis
        if set_fig is not None:
            self.fig = set_fig
        if set_axes is not None:
            self.axes = set_axes

        if self.fig is None:
            self.fig = plt.figure()

        if self.axes is None:
            self.axes = np.array([plt.axes()])
            self.fig.add_axes(self.axes[0])

        # Calculate u/v wind components from speed/direction
        if uv_flag is False:
            spd = self._ds[dsname][spd_field].values * units(
                self._ds[dsname][spd_field].attrs['units']
            )
            dir = self._ds[dsname][dir_field].values * units(
                self._ds[dsname][dir_field].attrs['units']
            )
            u, v = mpcalc.wind_components(spd, dir)
        else:
            u = self._ds[dsname][spd_field].values * units(
                self._ds[dsname][spd_field].attrs['units']
            )
            v = self._ds[dsname][dir_field].values * units(
                self._ds[dsname][dir_field].attrs['units']
            )

        # Plot out the data using the Hodograph method
        h = Hodograph(self.axes, component_range=component_range)
        h.add_grid(increment=20)
        if color_field is None:
            h.plot(u, v)
        else:
            data = self._ds[dsname][color_field].values * units(
                self._ds[dsname][color_field].attrs['units']
            )
            h.plot_colormapped(u, v, data)

        return self.axes

    def add_stability_info(
        self,
        temp_name='tdry',
        td_name='dp',
        p_name='pres',
        overwrite_data=None,
        add_data=None,
        set_fig=None,
        set_axes=None,
        dsname=None,
    ):
        """
        This plot will make a sounding plot from wind data that is given
        in speed and direction.

        Parameters
        ----------
        temp_name : str
            The name of the temperature field.
        td_name : str
            The name of the dewpoint field.
        p_name : str
            The name of the pressure field.
        overwrite_data : dict
            A disctionary of variables/values to write out instead
            of the ones calculated by MetPy.  Needs to be of the form
            .. code-block:: python

                overwrite_data={'LCL': 234, 'CAPE': 25}
                ...
        add_data : dict
            A dictionary of variables and values to write out in
            addition to the MetPy calculated ones
        set_fig : matplotlib figure, optional
            The figure to plot on
        set_axes : matplotlib axes, optional
            The specific axes to plot on
        dsname : str
             Name of the datastream to plot if multiple in the plot object

        Returns
        -------
        self.axes : matplotlib axes

        """

        if dsname is None and len(self._ds.keys()) > 1:
            raise ValueError(
                'You must choose a datastream when there are 2 '
                'or more datasets in the TimeSeriesDisplay '
                'object.'
            )
        elif dsname is None:
            dsname = list(self._ds.keys())[0]

        # Get the current plotting axis
        if set_fig is not None:
            self.fig = set_fig
        if set_axes is not None:
            self.axes = set_axes

        if self.fig is None:
            self.fig = plt.figure()

        if self.axes is None:
            self.axes = np.array([plt.axes()])
            self.fig.add_axes(self.axes[0])

        self.axes.spines['top'].set_visible(False)
        self.axes.spines['right'].set_visible(False)
        self.axes.spines['bottom'].set_visible(False)
        self.axes.spines['left'].set_visible(False)
        self.axes.get_xaxis().set_ticks([])
        self.axes.get_yaxis().set_ticks([])
        ct = 0
        if overwrite_data is None:
            # Calculate stability indicies
            ds_sonde = calculate_stability_indicies(
                self._ds[dsname],
                temp_name=temp_name,
                td_name=td_name,
                p_name=p_name,
            )

            # Add MetPy calculated variables to the list
            variables = {
                'lifted_index': 'Lifted Index',
                'surface_based_cape': 'SBCAPE',
                'surface_based_cin': 'SBCIN',
                'most_unstable_cape': 'MUCAPE',
                'most_unstable_cin': 'MUCIN',
                'lifted_condensation_level_temperature': 'LCL Temp',
                'lifted_condensation_level_pressure': 'LCL Pres',
            }
            for i, v in enumerate(variables):
                var_string = str(np.round(ds_sonde[v].values, 2))
                self.axes.text(
                    -0.05,
                    (0.98 - (0.1 * i)),
                    variables[v] + ': ',
                    transform=self.axes.transAxes,
                    fontsize=10,
                    verticalalignment='top',
                )
                self.axes.text(
                    0.95,
                    (0.98 - (0.1 * i)),
                    var_string,
                    transform=self.axes.transAxes,
                    fontsize=10,
                    verticalalignment='top',
                    horizontalalignment='right',
                )
                ct += 1
        else:
            # If overwrite_data is set, the user passes in their own dictionary
            for i, v in enumerate(overwrite_data):
                var_string = str(np.round(overwrite_data[v], 2))
                self.axes.text(
                    -0.05,
                    (0.98 - (0.1 * i)),
                    v + ': ',
                    transform=self.axes.transAxes,
                    fontsize=10,
                    verticalalignment='top',
                )
                self.axes.text(
                    0.95,
                    (0.98 - (0.1 * i)),
                    var_string,
                    transform=self.axes.transAxes,
                    fontsize=10,
                    verticalalignment='top',
                    horizontalalignment='right',
                )
        # User can also add variables to the existing ones calculated by MetPy
        if add_data is not None:
            for i, v in enumerate(add_data):
                var_string = str(np.round(add_data[v], 2))
                self.axes.text(
                    -0.05,
                    (0.98 - (0.1 * (i + ct))),
                    v + ': ',
                    transform=self.axes.transAxes,
                    fontsize=10,
                    verticalalignment='top',
                )
                self.axes.text(
                    0.95,
                    (0.98 - (0.1 * (i + ct))),
                    var_string,
                    transform=self.axes.transAxes,
                    fontsize=10,
                    verticalalignment='top',
                    horizontalalignment='right',
                )
        return self.axes

    def plot_enhanced_skewt(
        self,
        spd_name='wspd',
        dir_name='deg',
        temp_name='tdry',
        td_name='dp',
        p_name='pres',
        overwrite_data=None,
        add_data=None,
        color_field=None,
        component_range=80,
        uv_flag=False,
        dsname=None,
        figsize=(14, 10),
        layout='constrained',
    ):
        """
        This will plot an enhanced Skew-T plot with a Hodograph on the top right
        and the stability parameters on the lower right.  This will create a new
        figure so that one does not need to be defined through subplot_shape.

        Requires Matplotlib v 3.7 and higher

        Parameters
        ----------
        spd_name : str
            The name of the field corresponding to the wind speed.
        dir_name : str
            The name of the field corresponding to the wind direction
            in degrees from North.
        temp_name : str
            The name of the temperature field.
        td_name : str
            The name of the dewpoint field.
        p_name : str
            The name of the pressure field.
        overwrite_data : dict
            A disctionary of variables/values to write out instead
            of the ones calculated by MetPy.  Needs to be of the form
            .. code-block:: python

                overwrite_data={'LCL': 234, 'CAPE': 25}
                ...
        add_data : dict
            A dictionary of variables and values to write out in
            addition to the MetPy calculated ones
        color_field : str, optional
            The name of the field if wanting to shade by another variable
        component_range : int
             Range of the hodograph.  Default is 80
        uv_flag : boolean
             If set to True, spd_field and dir_field will be treated as the
             U and V wind variable names
        dsname : str
             Name of the datastream to plot if multiple in the plot object
        figsize : tuple
             Figure size for the plot
        layout : str
            String to pass to matplotlib.figure.Figure object layout keyword
            argument. Choice of 'constrained,' 'compressed,' 'tight,' or None.
            Default is 'constrained'.

        Returns
        -------
        self.axes : matplotlib axes

        """

        # Set up the figure and axes
        # Close existing figure as a new one will be created
        plt.close('all')
        subplot_kw = {'a': {'projection': 'skewx'}}
        fig, axs = plt.subplot_mosaic(
            [['a', 'a', 'b'], ['a', 'a', 'b'], ['a', 'a', 'c'], ['a', 'a', 'c']],
            layout=layout,
            per_subplot_kw=subplot_kw,
        )
        self.fig = fig
        self.axes = axs

        # Plot out the Skew-T
        display = SkewTDisplay(self._ds, set_fig=fig, subplot=axs['a'], figsize=figsize)
        if uv_flag is True:
            display.plot_from_u_and_v(spd_name, dir_name, p_name, temp_name, td_name)
        else:
            display.plot_from_spd_and_dir(spd_name, dir_name, p_name, temp_name, td_name)

        # Plot the hodograph
        display.plot_hodograph(
            spd_name,
            dir_name,
            set_axes=axs['b'],
            color_field=color_field,
            component_range=component_range,
            dsname=dsname,
            uv_flag=uv_flag,
        )

        # Add Stability information
        display.add_stability_info(
            set_axes=axs['c'],
            temp_name=temp_name,
            td_name=td_name,
            p_name=p_name,
            overwrite_data=overwrite_data,
            add_data=add_data,
            dsname=dsname,
        )
        return self.axes



"""
Multidimensional cross sections
-------------------------------

In this example, the VISST data are used to
plot up cross-sectional slices through the
multi-dimensional dataset
"""


from arm_test_data import DATASETS
import matplotlib.pyplot as plt

import act

filename_visst = DATASETS.fetch('twpvisstgridirtemp.c1.20050705.002500.nc')
my_ds = act.io.arm.read_arm_netcdf(filename_visst)

# Cross section display requires that the variable being plotted be reduced to two
# Dimensions whose coordinates can be specified by variables in the file
display = act.plotting.XSectionDisplay(my_ds, figsize=(20, 8), subplot_shape=(2, 2))
display.plot_xsection_map(
    'ir_temperature',
    None,
    x='longitude',
    y='latitude',
    cmap='Greys',
    vmin=200,
    vmax=320,
    subplot_index=(0, 0),
)
display.plot_xsection_map(
    'ir_temperature',
    None,
    x='longitude',
    y='latitude',
    cmap='Greys',
    vmin=200,
    vmax=320,
    subplot_index=(1, 0),
)
display.plot_xsection_map(
    'ir_temperature',
    None,
    x='longitude',
    y='latitude',
    cmap='Greys',
    vmin=200,
    vmax=320,
    subplot_index=(0, 1),
)
display.plot_xsection_map(
    'ir_temperature',
    None,
    x='longitude',
    y='latitude',
    cmap='Greys',
    vmin=200,
    vmax=320,
    subplot_index=(1, 1),
)

plt.show()
my_ds.close()



"""
Read and plot a PPI from raw mini-MPL data
------------------------------------------

Example of how to read in raw data from the mini-MPL
and plot out the PPI by converting it to PyART

Author: Adam Theisen
"""

from arm_test_data import DATASETS
from matplotlib import pyplot as plt

import act

try:
    import pyart

    PYART_AVAILABLE = True
except ImportError:
    PYART_AVAILABLE = False

# Read in sample mini-MPL data
filename_mpl = DATASETS.fetch('201509021500.bi')
ds = act.io.mpl.read_sigma_mplv5(filename_mpl)

# Create a PyART Radar Object
radar = act.utils.create_pyart_obj(
    ds, azimuth='azimuth_angle', elevation='elevation_angle', range_var='range'
)

# Creat Plot Display
if PYART_AVAILABLE:
    display = pyart.graph.RadarDisplay(radar)
    display.plot('nrb_copol', sweep=0, title_flag=False, vmin=0, vmax=1.0, cmap='jet')
    plt.show()



import numpy as np
import pytest

from act.qc.sp2 import PYSP2_AVAILABLE, SP2ParticleCriteria


@pytest.mark.skipif(not PYSP2_AVAILABLE, reason='PySP2 is not installed.')
def test_sp2_particle_config():
    particle_config_ds = SP2ParticleCriteria()
    assert particle_config_ds.ScatMaxPeakHt1 == 60000
    assert particle_config_ds.ScatMinPeakHt1 == 250
    assert particle_config_ds.ScatMaxPeakHt2 == 60000
    assert particle_config_ds.ScatMinPeakHt2 == 250
    assert particle_config_ds.ScatMinWidth == 10
    assert particle_config_ds.ScatMaxWidth == 90
    assert particle_config_ds.ScatMinPeakPos == 20
    assert particle_config_ds.ScatMaxPeakPos == 90
    assert particle_config_ds.IncanMinPeakHt1 == 200
    assert particle_config_ds.IncanMinPeakHt2 == 200
    assert particle_config_ds.IncanMaxPeakHt1 == 60000
    assert particle_config_ds.IncanMaxPeakHt2 == 60000
    assert particle_config_ds.IncanMinWidth == 5
    assert particle_config_ds.IncanMaxWidth == np.inf
    assert particle_config_ds.IncanMinPeakPos == 20
    assert particle_config_ds.IncanMaxPeakPos == 90
    assert particle_config_ds.IncanMinPeakRatio == 0.1
    assert particle_config_ds.IncanMaxPeakRatio == 25
    assert particle_config_ds.IncanMaxPeakOffset == 11
    assert particle_config_ds.c0Mass1 == 0
    assert particle_config_ds.c1Mass1 == 0.0001896
    assert particle_config_ds.c2Mass1 == 0
    assert particle_config_ds.c3Mass1 == 0
    assert particle_config_ds.c0Mass2 == 0
    assert particle_config_ds.c1Mass2 == 0.0016815
    assert particle_config_ds.c2Mass2 == 0
    assert particle_config_ds.c3Mass2 == 0
    assert particle_config_ds.c0Scat1 == 0
    assert particle_config_ds.c1Scat1 == 78.141
    assert particle_config_ds.c2Scat1 == 0
    assert particle_config_ds.c0Scat2 == 0
    assert particle_config_ds.c1Scat2 == 752.53
    assert particle_config_ds.c2Scat2 == 0
    assert particle_config_ds.densitySO4 == 1.8
    assert particle_config_ds.densityBC == 1.8
    assert particle_config_ds.TempSTP == 273.15
    assert particle_config_ds.PressSTP == 1013.25



import fsspec
import numpy as np

import act


def test_read_sodar():
    ds = act.io.read_mfas_sodar(act.tests.EXAMPLE_MFAS_SODAR)

    # Test coordinates.
    assert ds.time.shape[0] == 96
    assert ds.time[0].dtype == 'datetime64[ns]'

    assert ds.height.shape[0] == 58
    assert ds.height[0] == 30.0
    assert ds.height[-1] == 600.0

    # Test variable data, shape and attributes.
    assert len(ds.data_vars) == 26
    assert ds['dir'].shape == (96, 58)
    direction = ds['dir'][0, 0:5].values
    np.testing.assert_allclose(direction, [129.9, 144.2, 147.5, 143.5, 143.0], rtol=1e-6)
    pgz = ds['PGz'][0, 0:5].values
    np.testing.assert_allclose(pgz, [4, 4, 4, 5, 5])

    assert ds['dir'].attrs['variable_name'] == 'wind direction'
    assert ds['dir'].attrs['symbol'] == 'dir'
    assert ds['dir'].attrs['type'] == 'R1'
    assert ds['dir'].attrs['_FillValue'] == 999.9
    assert ds['dir'].attrs['error_mask'] == '0'
    assert ds['dir'].attrs['units'] == 'deg'

    # Test global attributes.
    assert ds.attrs['height above sea level [m]'] == 0.0
    assert ds.attrs['instrument_type'] == 'MFAS'


def test_metadata_retrieval():
    # Read the file and lines.
    file = fsspec.open(act.tests.EXAMPLE_MFAS_SODAR).open()
    lines = file.readlines()
    lines = [x.decode().rstrip()[:] for x in lines]

    # Retrieve metadata.
    file_dict, variable_dict = act.io.sodar._metadata_retrieval(lines)

    # Test file dictionary.
    assert 'instrument_type' in file_dict
    assert 'height above sea level [m]' in file_dict

    assert file_dict['format'] == 'FORMAT-1'
    assert file_dict['height above ground [m]'] == 0.0

    # Test variable dictionary.
    assert 'speed' in variable_dict.keys()
    assert 'error' not in variable_dict.keys()

    assert 'variable_name' in variable_dict['sigSpeed']
    assert 'units' in variable_dict['sigSpeed']
    assert '_FillValue' in variable_dict['sigSpeed']

    assert variable_dict['W']['units'] == 'm/s'
    assert variable_dict['W']['variable_name'] == 'wind W'
    assert variable_dict['W']['_FillValue'] == 99.99



"""
Data rose plot
--------------

This is an example of how to display a data rose.
As can be seen in the final plot, there are two major
bullseyes of data, one around 0ºC to the Northeast and
another around 15ºC to the South. This tells us that we
get lower temperatures when winds are out of the N/NE as
would be expected at this location.  This can be extended
to easily review other types of data as well like aerosols
and fluxes.

"""

from arm_test_data import DATASETS
from matplotlib import pyplot as plt

import act

# Read in some data with wind speed/direction in the file
met_wildcard_list = [
    'sgpmetE13.b1.20190101.000000.cdf',
    'sgpmetE13.b1.20190102.000000.cdf',
    'sgpmetE13.b1.20190103.000000.cdf',
    'sgpmetE13.b1.20190104.000000.cdf',
    'sgpmetE13.b1.20190105.000000.cdf',
    'sgpmetE13.b1.20190106.000000.cdf',
    'sgpmetE13.b1.20190107.000000.cdf',
]
met_filenames = [DATASETS.fetch(file) for file in met_wildcard_list]
ds = act.io.arm.read_arm_netcdf(met_filenames)

# Set up wind rose display object
display = act.plotting.WindRoseDisplay(ds, subplot_shape=(2, 3), figsize=(16, 10))

# Plot mean temperature based on wind direction
display.plot_data(
    'wdir_vec_mean',
    'wspd_vec_mean',
    'temp_mean',
    num_dirs=12,
    plot_type='line',
    subplot_index=(0, 0),
)

# Plot median temperature based on wind direction
display.plot_data(
    'wdir_vec_mean',
    'wspd_vec_mean',
    'temp_mean',
    num_dirs=12,
    plot_type='line',
    subplot_index=(0, 1),
    line_plot_calc='median',
)

# Plot standard deviation of temperature based on wind direction
display.plot_data(
    'wdir_vec_mean',
    'wspd_vec_mean',
    'temp_mean',
    num_dirs=12,
    plot_type='line',
    subplot_index=(0, 2),
    line_plot_calc='stdev',
)

# Plot a contour of counts of temperature based on wind direction
display.plot_data(
    'wdir_vec_mean',
    'wspd_vec_mean',
    'temp_mean',
    num_dirs=12,
    plot_type='contour',
    subplot_index=(1, 0),
)

# Plot a contour of mean temperature based on wind direction and wind speed
display.plot_data(
    'wdir_vec_mean',
    'wspd_vec_mean',
    'temp_mean',
    num_dirs=12,
    plot_type='contour',
    contour_type='mean',
    num_data_bins=10,
    clevels=21,
    cmap='rainbow',
    vmin=-5,
    vmax=20,
    subplot_index=(1, 1),
)

# Plot a boxplot of temperature based on wind direction
display.plot_data(
    'wdir_vec_mean',
    'wspd_vec_mean',
    'temp_mean',
    num_dirs=12,
    plot_type='boxplot',
    subplot_index=(1, 2),
)

plt.show()



"""
Correct wind data for ship motion
---------------------------------------------------

This example shows how to calculate course and speed
over ground of the ship and use it to correct the
wind speed and direction data.
"""
from arm_test_data import DATASETS
import xarray as xr

import act

# Read in the navigation data, mainly for the lat/lon
filename_nav = DATASETS.fetch('marnavM1.a1.20180201.000000.nc')
nav_ds = act.io.arm.read_arm_netcdf(filename_nav)

# Calculate course and speed over ground from the NAV
# lat and lon data
nav_ds = act.utils.ship_utils.calc_cog_sog(nav_ds)

# Read in the data containing the wind speed and direction
filename_aosmet = DATASETS.fetch('maraosmetM1.a1.20180201.000000.nc')
aosmet_ds = act.io.arm.read_arm_netcdf(filename_aosmet)

# Merge the navigation and wind data together
# This have been previously resampled to 1-minute data
ds = xr.merge([nav_ds, aosmet_ds], compat='override')

# Call the correction for the winds.  Note, that this only
# corrects for ship course and speed, not roll and pitch.
ds = act.corrections.ship.correct_wind(ds)

nav_ds.close()
aosmet_ds.close()
ds.close()



import yaml
import numpy as np
from pathlib import Path
from dateutil import parser

#   Example of the YAML file and how to construct.
#   The times are set as inclusive start to inclusive end time.
#   Different formats are acceptable as displayed with temp_mean
#   Good example below.
#
#   The general format is to use the variable name as initial key
#   followed by an assessment key, followed by a 'description' key which
#   is a description of that test. Each test listed will insert a new test
#   into the ancillary quality control variable. After the 'description'
#   list all time ranges as a YAML array.
#   If a test is to be applied to all variables list under the specail
#   variable name '_all'.
#
#   The file should be named same as datastream name with standard
#   YAML file extension when only providing the directoy.
#   For example the file below would be named sgpmetE13.b1.yaml
#
#
#  _all:
#    Bad:
#      Values are bad for all:
#        - 2020-01-21 01:01:02, 2020-01-21 01:03:13
#        - 2020-01-21 02:01:02, 2020-01-21 04:03:13
#   Suspect:
#     Values are suspect for all: []
#
# temp_mean:
#   Bad:
#     Values are bad:
#       - 2020-01-01 00:01:02, 2020-01-01 00:03:44
#       - 2020-01-02 01:01:02, 2020-01-02 01:03:13
#       - 2020-02-01 00:01:02, 2020-02-01 00:03:44
#       - 2020-03-02 01:01:02, 2020-03-02 01:03:13
#       - 2020-01-21 01:01:02, 2020-01-21 01:03:13
#   Suspect:
#     Values are suspect:
#       - 2020-01-01 02:04:02, 2020-01-01 02:05:44
#   Good:
#     Values are good:
#       - 2020-01-01 00:08:02, 2020-01-01 00:09:44
#       - Jan 1, 2020 00:08:02 ; January 1, 2020 00:09:44 AM
#       - 2020-01-01 00:08 | 2020-01-01 00:09
#       - 2020-01-01T00:08:02 ; 2020-01-01T00:09:44
#
# rh_mean:
#   Bad:
#     Values are bad:
#       - 2020-01-01 00:01:02, 2020-01-01 00:03:44
#       - 2020-01-02 00:01:02, 2020-01-02 00:03:44
#   Suspect:
#     Values are suspect:
#       - 2020-01-01 00:04:02, 2020-01-01 00:05:44


def read_yaml_supplemental_qc(
    ds,
    fullpath,
    variables=None,
    assessments=None,
    datetime64=True,
    time_delim=(';', ',', '|', r'\t'),
    none_if_empty=True,
    quiet=False,
):
    """
    Returns a dictionary converstion of YAML file for flagging data. The dictionary
    will contain variable names as first key, assessents as second keys containing
    test description as third key with time as last key. Multiple descriptions
    are allowed.

    Parameters
    ----------
    ds : xarray.Dataset
        Xarray dataset containing data.
    fullpath : str or `pathlib.Path`
        Full path to file to read or directory containing YAML files to read. If providing
        a directory a file with the datastream name ending in .yaml or .yml must exists.
    variables : str, list of str or None
        Optional, variable names to keep in dictionary. All others will be removed
        before returning dictionary. If no variables are left will return None.
    assessments : str, list of str or None
        Optional, assessments categories to keep in dictionary. All others will be removed
        before returning dictionary.
    datetime64 : boolean
        Convert the string value list to 2D numpy datetime64 array with first value
        the start time and second value end time. If the time list in the YAML file
        is empty, the 'time' numpy array will be a single value instead of 2D array
        and set to numpy datetime64 NaT.
    time_delim : str or list of str or tuple of str
        Optional, character delimiter to use when parsing times.
    none_if_empty : boolean
        Return None instead of empty dictionary
    quiet : boolean
        Suppress information about not finding a YAML file to read.

    Returns
    -------
        Dictionary of [variable names][assessments][description] and time values
        or if the dictionary is empty after processing options and none_if_empty set
        to True will return None.

    Examples
    --------
    This example will load the example MET data used for unit testing.

    .. code-block:: python

        from act.tests import EXAPLE_MET_YAML, EXAMPLE_MET1
        from act.io.arm import read_arm_netcdf
        from act.qc.add_supplemental_qc import read_yaml_supplemental_qc
        ds = read_arm_netcdf(EXAMPLE_MET1, cleanup_qc=True)
        result = read_yaml_supplemental_qc(ds, EXAPLE_MET_YAML,
                                     variables=['rh_mean'], assessments='Bad')
        print(result)

        {'rh_mean': {'Bad': {'description': 'Values are bad',
            'time': array([['2020-01-01T00:01:02.000', '2020-01-01T00:03:44.000'],
                           ['2020-01-02T00:01:02.000', '2020-01-02T00:03:44.000']],
                           dtype='datetime64[ms]')}}}

    """

    flag_file = []
    if Path(fullpath).is_file():
        flag_file = [Path(fullpath)]
    else:
        try:
            datastream = ds.attrs['_datastream']
        except KeyError:
            raise RuntimeError(
                'Unable to determine datastream name from Dataset. Need to set global attribute '
                '_datastream in Dataset or provided full path to flag file.'
            )

        flag_file = list(Path(fullpath).glob(f'{datastream}.yml'))
        flag_file.extend(list(Path(fullpath).glob(f'{datastream}.yaml')))

    if len(flag_file) > 0:
        flag_file = flag_file[0]
    else:
        if not quiet:
            print(f'Could not find supplemental QC file for {datastream} in {fullpath}')

        return None

    # Ensure keywords are lists
    if isinstance(variables, str):
        variables = [variables]

    if isinstance(assessments, str):
        assessments = [assessments]

    if isinstance(time_delim, str):
        time_delim = [time_delim]

    # Ensure the assessments are capitalized for matching
    if assessments is not None:
        assessments = [ii.capitalize() for ii in assessments]

    # Read YAML file
    with open(flag_file) as fp:
        try:
            data_dict = yaml.load(fp, Loader=yaml.FullLoader)
        except AttributeError:
            data_dict = yaml.load(fp)

    # If variable names are provided only keep those names.
    if variables is not None:
        variables.append('_all')
        del_vars = set(data_dict.keys()) - set(variables)
        for var_name in del_vars:
            del data_dict[var_name]

    # If assessments are given only keep those assessments.
    if assessments is not None:
        for var_name in data_dict.keys():
            for asses_name in data_dict[var_name].keys():
                # Check if yaml file assessments are capitalized. If not fix.
                if not asses_name == asses_name.capitalize():
                    data_dict[var_name][asses_name.capitalize()] = data_dict[var_name][asses_name]
                    del data_dict[var_name][asses_name]

            # Delete assessments if not in provided list.
            del_asses = set(data_dict[var_name].keys()) - set(assessments)
            for asses_name in del_asses:
                del data_dict[var_name][asses_name]

    # Convert from string to numpy datetime64 2D array
    if datetime64:
        for var_name in data_dict.keys():
            for asses_name in data_dict[var_name].keys():
                for description in data_dict[var_name][asses_name].keys():
                    try:
                        num_times = len(data_dict[var_name][asses_name][description])
                        new_times = np.empty((num_times, 2), dtype='datetime64[ms]')
                    except TypeError:
                        # Set to single value Not A Time numpy array if times
                        # from yaml file are empty list.
                        new_times = np.full([], np.datetime64('NaT'), dtype='datetime64[ms]')

                    for ii, tm in enumerate(data_dict[var_name][asses_name][description]):
                        # Split the times on multiple different types of delimiters.
                        for delim in time_delim:
                            split_tm = tm.split(delim)
                            if len(split_tm) > 1:
                                break

                        new_times[ii, 0] = np.datetime64(parser.parse(split_tm[0]))
                        new_times[ii, 1] = np.datetime64(parser.parse(split_tm[1]))

                    data_dict[var_name][asses_name][description] = new_times

    # If the return dictinary is empty convert to None
    if none_if_empty and len(data_dict) == 0:
        data_dict = None

    return data_dict


def apply_supplemental_qc(
    ds,
    fullpath,
    variables=None,
    assessments=None,
    apply_all=True,
    exclude_all_variables=None,
    quiet=False,
):
    """
    Apply flagging from supplemental QC file by adding new QC tests.

    Parameters
    ----------
    ds : xarray.Dataset
        Xarray dataset containing data. QC variables should be converted to CF
        format prior to adding new tests.
    fullpath : str or `pathlib.Path`
        Fullpath to file or directory with supplemental QC files.
    variables : str, list of str or None
        Variables to apply to the dataset from supplemental QC flag file.
        If not set will apply all variables in the file.
    assessments : str, list of str or None
        Assessments to apply. If not not set will apply all assesments in the flag file.
    apply_all : boolean
        If a "_all" variable exists in the supplemental QC flag file will apply to all variables
        in the Dataset.
    exclude_all_variables : str, list of str or None
        Variables to skip when applying "_all" variables.
    quiet : boolean
        Suppress information about not finding a supplemental QC file to read.

    Examples
    --------
    This example will load the example sounding data used for unit testing.

    .. code-block:: python

        from act.tests import EXAPLE_MET_YAML, EXAMPLE_MET1
        from act.io.arm import read_arm_netcdf
        from act.qc.add_supplemental_qc import apply_supplemental_qc
        ds = read_arm_netcdf(EXAMPLE_MET1, cleanup_qc=True)
        apply_supplemental_qc(ds, EXAPLE_MET_YAML, apply_all=False)
        print(ds['qc_temp_mean'].attrs['flag_meanings'])

        ['Value is equal to missing_value.', 'Value is less than the fail_min.',
         'Value is greater than the fail_max.',
         'Difference between current and previous values exceeds fail_delta.',
         'Values are bad', 'Values are super bad', 'Values are suspect', 'Values are good']

    """

    exclude_vars = ['time', 'base_time', 'time_offset']
    if exclude_all_variables is not None:
        if isinstance(exclude_all_variables, str):
            exclude_all_variables = [exclude_all_variables]

        exclude_vars.extend(exclude_all_variables)

    flag_dict = read_yaml_supplemental_qc(
        ds, fullpath, variables=variables, assessments=assessments, quiet=quiet
    )

    if flag_dict is None:
        return

    for var_name in list(ds.variables):
        if var_name in flag_dict.keys():
            for asses_name in flag_dict[var_name].keys():
                for description in flag_dict[var_name][asses_name]:
                    times = flag_dict[var_name][asses_name][description]

                    if np.all(np.isnat(times)):
                        continue

                    indexes = np.array([], dtype=np.int32)
                    for vals in times:
                        ind = np.argwhere(
                            (ds['time'].values >= vals[0]) & (ds['time'].values <= vals[1])
                        )

                        if len(ind) > 0:
                            indexes = np.append(indexes, ind)

                    if indexes.size > 0:
                        ds.qcfilter.add_test(
                            var_name,
                            index=indexes,
                            test_meaning=description,
                            test_assessment=asses_name,
                        )

    var_name = '_all'
    if apply_all and var_name in flag_dict.keys():
        for asses_name in flag_dict[var_name].keys():
            for description in flag_dict[var_name][asses_name]:
                times = flag_dict[var_name][asses_name][description]

                if np.all(np.isnat(times)):
                    continue

                indexes = np.array([], dtype=np.int32)
                for vals in times:
                    ind = np.argwhere(
                        (ds['time'].values >= vals[0]) & (ds['time'].values <= vals[1])
                    )
                    if ind.size > 0:
                        indexes = np.append(indexes, np.ndarray.flatten(ind))

                if indexes.size > 0:
                    for all_var_name in list(ds.data_vars):
                        if all_var_name in exclude_vars:
                            continue

                        if 'time' not in ds[all_var_name].dims:
                            continue

                        try:
                            if ds[all_var_name].attrs['standard_name'] == 'quality_flag':
                                continue
                        except KeyError:
                            pass

                        ds.qcfilter.add_test(
                            all_var_name,
                            index=indexes,
                            test_meaning=description,
                            test_assessment=asses_name,
                        )



"""
Plot a histogram of Met data.
----------------------------------------------------

This is a simple example for how to plot a histogram
of Meteorological data, while using hist_kwargs parameter.

Author: Zachary Sherman
"""

from arm_test_data import DATASETS
from matplotlib import pyplot as plt
import numpy as np

import act

filename_met = DATASETS.fetch('sgpmetE13.b1.20190101.000000.cdf')
met_ds = act.io.arm.read_arm_netcdf(filename_met)

# Plot data
hist_kwargs = {'range': (-10, 10)}
histdisplay = act.plotting.DistributionDisplay(met_ds)
histdisplay.plot_stacked_bar('temp_mean', bins=np.arange(-40, 40, 5), hist_kwargs=hist_kwargs)
plt.show()



"""
Calculate and plot weighted means
---------------------------------

This is an example of how to calculate a
weighted average from the MET TBRG, ORG
and PWD.  This also calculates the
accumulated precipitation and displays it

"""

from arm_test_data import DATASETS
import matplotlib.pyplot as plt
import xarray as xr

import act

# Specify dictionary of datastreams, variables, and weights
# Note, all weights should add up to 1.
cf_ds = {
    'sgpmetE13.b1': {
        'variable': [
            'tbrg_precip_total',
            'org_precip_rate_mean',
            'pwd_precip_rate_mean_1min',
        ],
        'weight': [0.8, 0.15, 0.05],
    }
}

# Other way to define cf_ds
# cf_ds = {'sgpmetE13.b1': {'variable': ['tbrg_precip_total'], 'weight': [0.5]},
#         'sgpmetE13.b1': {'variable': ['org_precip_rate_mean'], 'weight': [0.25]},
#         'sgpmetE13.b1': {'variable': ['pwd_precip_rate_mean_1min'], 'weight': [0.25]}
#        }

# Get a list of filenames to use
met_wildcard_list = [
    'sgpmetE13.b1.20190101.000000.cdf',
    'sgpmetE13.b1.20190102.000000.cdf',
    'sgpmetE13.b1.20190103.000000.cdf',
    'sgpmetE13.b1.20190104.000000.cdf',
    'sgpmetE13.b1.20190105.000000.cdf',
    'sgpmetE13.b1.20190106.000000.cdf',
    'sgpmetE13.b1.20190107.000000.cdf',
]

ds = {}
new = {}
out_units = 'mm/hr'
for d in cf_ds:
    met_filenames = [DATASETS.fetch(file) for file in met_wildcard_list]
    ds = act.io.arm.read_arm_netcdf(met_filenames)
    # Loop through each variable and add to data list
    new_da = []
    for v in cf_ds[d]['variable']:
        da = ds[v]
        # Accumulate precip variables in new dataset
        ds = act.utils.data_utils.accumulate_precip(ds, v)

        # Convert units and add to dataarray list
        units = da.attrs['units']
        if units == 'mm':
            da.attrs['units'] = 'mm/min'
        da.values = act.utils.data_utils.convert_units(da.values, da.attrs['units'], out_units)
        da = da.resample(time='1min').mean()
        new_da.append(da)

    # Depending on number of variables for each datastream, merge or create dataset
    if len(new_da) > 1:
        new_da = xr.merge(new_da)
    else:
        new_da = new_da[0].to_dataset()

    # Add to dictionary for the weighting
    cf_ds[d]['ds'] = new_da

    # Add dataset to dictionary for plotting
    new[d] = ds

# Calculate weighted averages using the dict defined above
data = act.utils.data_utils.ts_weighted_average(cf_ds)

# Add weighted mean to plotting object and calculate accumulation
new['weighted'] = data.to_dataset(name='weighted_mean')
new['weighted']['weighted_mean'].attrs['units'] = 'mm/hr'
new['weighted'] = act.utils.data_utils.accumulate_precip(new['weighted'], 'weighted_mean')

# Plot the accumulations
display = act.plotting.TimeSeriesDisplay(new, figsize=(12, 8), subplot_shape=(1,))
display.plot('tbrg_precip_total_accumulated', dsname='sgpmetE13.b1', color='b', label='TBRG, 0.8')
display.plot(
    'org_precip_rate_mean_accumulated',
    dsname='sgpmetE13.b1',
    color='g',
    label='ORG 0.15',
)
display.plot(
    'pwd_precip_rate_mean_1min_accumulated',
    dsname='sgpmetE13.b1',
    color='y',
    label='PWD 0.05',
)
display.plot('weighted_mean_accumulated', dsname='weighted', color='k', label='Weighted Avg')
display.day_night_background('sgpmetE13.b1')
display.axes[0].legend()
plt.show()



"""
Function for downloading data from the NASA Atmospheric Science Data Center
(ASDC), which hosts data including the Atmopsheric Composition
Ground Observation Network.

"""
import os
import requests
import re
import shutil


def download_mplnet_data(
    version=None,
    level=None,
    product=None,
    site=None,
    year=None,
    month=None,
    day=None,
    outdir=None,
):
    """
    Function to download data from the NASA MPL Network Data
    https://mplnet.gsfc.nasa.gov/mplnet_web_services.cgi?download

    Downloaded Products are contained within NETCDF-4, CF compliant files.

    Parameters
    ----------
    version : int
        MPLNet Dataset Version Number (2 or 3).
        All data from 2000 have been processed to Version 3.
        Information on the MPLNet Dataset Version can be found here:
        https://mplnet.gsfc.nasa.gov/versions.htm
    level : int
        MPLNet Product Levels (1, 15, 2).
        MPLNet Levels used to differentiate quality assurance screens.
        Information on the MPLNet Product levels can be found here:
        https://mplnet.gsfc.nasa.gov/product-info/
        Level 1 data should never be used for publication.
    product : str
        MPLNet Product (NRB, CLD, PBL, AER).
            NRB - Lidar signals; volume depolarization ratos, diagnostics
            CLD - Cloud Heights, thin cloud extinction and optical depths, cloud
                    phase
            AER - Aerosol heights; extinction, backscatter, and aerosol
                    depolarization ratio profiles; lidar ratio
            PBL - Surface-Attached Mixed Layer Top and estimated mixed layer AOD
        Information on the MPLNet Products can be found here:
        https://mplnet.gsfc.nasa.gov/product-info/
    year : str
        Four digit Year for desired product download (YYYY).
        Note Level 1 and 1.5 products are available for
        download the day after automated collection.
        Information on the MPLNet naming convention can be found here:
        https://mplnet.gsfc.nasa.gov/product-info/mplnet_file_name.htm
    month : str
        Two digit month for desired product download (MM).
    day : str
        Two digit desired day for product download (DD).
        If day not supplied, will download all data for month supplied
        in a zip file.
    site : str
        MPLNet four letter site identifier.
    outdir : str
        The output directory for the data. Set to None to make a folder in the
        current working directory with the same name as *datastream* to place
        the files in.

    Returns
    -------
    files : list
        Returns list of files retrieved.
    """

    # Generate the data policy agreement information
    print("\nPlease Review the MPLNET Data Policy Prior to Use of MPLNET Data")
    print("The MPLNET Data Policy can be found at:\n\thttps://mplnet.gsfc.nasa.gov/data-policy\n")

    # Generate the data acknowledgement statement, might require site information.
    print(
        "Please Include the Following Acknowledgements in Any Publication \nor"
        + " presentation of MPLNET data, regardless of co-authorship status:"
    )
    print(
        "\n\tThe MPLNET project is funded by the NASA Radiation Sciences Program"
        + " \n\tand Earth Observing System."
    )
    print(
        "\n\tWe thank the MPLNET (PI) for (its/theirs) effort in establishing"
        + " \n\tand maintaining sites.\n"
    )

    # Add header
    headers = {
        'user-agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.99 Safari/537.36',
    }

    # Define the base URL
    base_url = "https://mplnet.gsfc.nasa.gov/download?"

    # Add specific information to the base URL
    if version is None:
        raise ValueError("Please provide a MPLNet Product Version")
    else:
        base_url += "version=V" + str(version)

    if level is None:
        raise ValueError("Please provide a MPLNet Product Level")
    else:
        base_url += "&level=L" + str(level)

    if product is None:
        raise ValueError("Please provide a specific MPLNet Product identifer")
    else:
        base_url += "&product=" + str(product)

    if site is None:
        raise ValueError("Please provide a specific MPLNet site")
    else:
        base_url += "&site=" + str(site)

    if year is None:
        raise ValueError("Year of desired data download is required to download MPLNET data")
    else:
        base_url += "&year=" + str(year)

    if month is None:
        raise ValueError("Month of desired data download is required to download MPLNet data")
    else:
        base_url += "&month=" + str(month)

    if day:
        # Note: Day is not required for the MPLNet download
        base_url += "&day=" + str(day)

    # Construct output directory
    if outdir:
        # Output files to directory specified
        output_dir = os.path.join(outdir)
    else:
        # If no folder given, add MPLNET folder
        # to current working dir to prevent file mix-up
        output_dir = os.path.join(os.getcwd(), "MPLNET")

    if not os.path.isdir(output_dir):
        os.makedirs(output_dir)

    # Make a Request
    files = []
    with requests.get(base_url, stream=True, headers=headers) as r:
        fname = re.findall("filename=(.+)", r.headers['Content-Disposition'])
        # Check for successful file check
        if fname[0][1:-1] == "MPLNET_download_fail.txt":
            raise ValueError(
                "Failed MPLNET Download\n"
                + " File could not be found for the desired input parameters"
                + " for MPLNET Download API"
            )
        else:
            output_filename = os.path.join(output_dir, fname[0][1:-1])
            print("[DOWNLOADING] ", fname[0][1:-1])
            with open(output_filename, 'wb') as f:
                shutil.copyfileobj(r.raw, f)
                files.append(output_filename)

    return files


def get_mplnet_meta(
    sites=None, method=None, year=None, month=None, day=None, print_to_screen=False
):
    """
    Returns a list of meta data from the NASA MPL Network Data
    https://mplnet.gsfc.nasa.gov/mplnet_web_services.cgi?metadata


    Parameters
    ----------
    sites : str
        How to return MPLNET Site Information
            all        - produces output on all sites (active and inactive)
            active     - produces output file containing only active sites
                       (if year, month, or day are not set then uses today's date)
            inactive   - produces output file containing only inactive sites
                       (if year, month, or day are not set then uses today's date)
            planned    - produces output file containing only planned sites
            site_name  - produces output file containing only requested site
            collection - produces output file containing sites in pre-defined
                         collections (e.g. field campaigns or regions)
    year : str
        Four digit Year for desired product download (YYYY).
        Note Level 1 and 1.5 products are available for
        download the day after automated collection.
        Information on the MPLNet naming convention can be found here:
        https://mplnet.gsfc.nasa.gov/product-info/mplnet_file_name.htm
    month : str
        Two digit month for desired product download (MM).
    day : str
        Two digit desired day for product download (DD).
        If day not supplied, will download all data for month supplied
        in a zip file.
    method : str
        Method for returning JSON list of MPLNET GALION format parameters.
            station - returns GALION JSON with only station and PI contact info
            data - return GALION JSON with data elements, station, date and PI
                contact information
    print_to_screen : Boolean
        If true, print MPLNET site identifiers to screen
    """

    # Add headers
    headers = {
        'user-agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.99 Safari/537.36',
    }

    # Define the base URL
    base_url = "https://mplnet.gsfc.nasa.gov/operations/sites?api&format=galion"

    if sites is None:
        raise ValueError("Site Parameter is required to download MPLNET Meta Data")
    else:
        base_url += "&sites=" + str(sites)

    if method:
        base_url += "&method=" + str(method)

    if year:
        base_url += "&year=" + str(year)

    if month:
        base_url += "&month=" + str(month)

    if day:
        base_url += "&day=" + str(day)

    with requests.get(base_url, stream=True, headers=headers) as r:
        # Convert to JSON
        site_request = r.json()
        if print_to_screen:
            for i in range(len(site_request)):
                print(site_request[i]['id'])

    return site_request



"""
Retrieve stability indicies from a sounding
-------------------------------------------------------------

This example shows how to retrieve CAPE, CIN, and lifted index
from a sounding.
"""

import warnings

from arm_test_data import DATASETS

import act

warnings.filterwarnings('ignore')


def print_summary(ds, variables):
    for var_name in variables:
        print(f'{var_name}: {ds[var_name].values} ' f"units={ds[var_name].attrs['units']}")
    print()


filename_sonde = DATASETS.fetch('sgpsondewnpnC1.b1.20190101.053200.cdf')
sonde_ds = act.io.arm.read_arm_netcdf(filename_sonde)

sonde_ds = act.retrievals.calculate_stability_indicies(
    sonde_ds, temp_name='tdry', td_name='dp', p_name='pres'
)

variables = [
    'lifted_index',
    'surface_based_cape',
    'surface_based_cin',
    'most_unstable_cape',
    'most_unstable_cin',
    'lifted_condensation_level_temperature',
    'lifted_condensation_level_pressure',
]

print_summary(sonde_ds, variables)



"""
Query the ARM DQR webservice
----------------------------

This example shows how to query the ARM Data Quality Report (DQR) webservice,
to retrieve the machine readable DQR information and print it out to screen.

Author: Adam Theisen
"""

import act

# Pass in a datastream, start date, and end date to get all DQRs in that range
# This will print it in a user readable format
dqr = act.qc.print_dqr('sgp30ecorE14.b1', '20150101', '20190101')

# If you want something that can be copy and pasted into a spreadsheet
# set pretty_print=False
dqr = act.qc.print_dqr('sgp30ecorE14.b1', '20150101', '20190101', pretty_print=False)



import xarray as xr

import act


def test_correct_wind():
    nav = act.io.arm.read_arm_netcdf(act.tests.sample_files.EXAMPLE_NAV)
    nav = act.utils.ship_utils.calc_cog_sog(nav)

    aosmet = act.io.arm.read_arm_netcdf(act.tests.sample_files.EXAMPLE_AOSMET)

    ds = xr.merge([nav, aosmet], compat='override')
    ds = act.corrections.ship.correct_wind(ds)

    assert round(ds['wind_speed_corrected'].values[800]) == 5.0
    assert round(ds['wind_direction_corrected'].values[800]) == 92.0



import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import pytest

import act
from act.plotting import SkewTDisplay
from act.tests import sample_files

matplotlib.use('Agg')


@pytest.mark.mpl_image_compare(tolerance=10)
def test_skewt_plot():
    sonde_ds = act.io.arm.read_arm_netcdf(sample_files.EXAMPLE_SONDE1)
    skewt = SkewTDisplay(sonde_ds)
    skewt.plot_from_u_and_v('u_wind', 'v_wind', 'pres', 'tdry', 'dp')
    sonde_ds.close()
    return skewt.fig


@pytest.mark.mpl_image_compare(tolerance=10)
def test_skewt_plot_spd_dir():
    sonde_ds = act.io.arm.read_arm_netcdf(sample_files.EXAMPLE_SONDE1)
    skewt = SkewTDisplay(sonde_ds, ds_name='act_datastream')
    skewt.plot_from_spd_and_dir('wspd', 'deg', 'pres', 'tdry', 'dp')
    sonde_ds.close()
    return skewt.fig


@pytest.mark.mpl_image_compare(tolerance=10)
def test_multi_skewt_plot():
    files = sample_files.EXAMPLE_TWP_SONDE_20060121
    test = {}
    for f in files:
        time = f.split('.')[-3]
        sonde_ds = act.io.arm.read_arm_netcdf(f)
        sonde_ds = sonde_ds.resample(time='30s').nearest()
        test.update({time: sonde_ds})

    skewt = SkewTDisplay(test, subplot_shape=(2, 2), figsize=(12, 14))
    i = 0
    j = 0
    for f in files:
        time = f.split('.')[-3]
        skewt.plot_from_spd_and_dir(
            'wspd',
            'deg',
            'pres',
            'tdry',
            'dp',
            subplot_index=(j, i),
            dsname=time,
            p_levels_to_plot=np.arange(10.0, 1000.0, 25),
        )
        skewt.axes[j, i].set_ylim([1000, 10])
        if j == 1:
            i += 1
            j = 0
        elif j == 0:
            j += 1
    plt.tight_layout()
    return skewt.fig


@pytest.mark.mpl_image_compare(tolerance=10)
def test_enhanced_skewt_plot():
    ds = act.io.arm.read_arm_netcdf(sample_files.EXAMPLE_SONDE1)
    display = act.plotting.SkewTDisplay(ds)
    with pytest.warns():
        display.plot_enhanced_skewt(color_field='alt', component_range=85)
    ds.close()
    return display.fig


@pytest.mark.mpl_image_compare(tolerance=10)
def test_enhanced_skewt_plot_2():
    ds = act.io.arm.read_arm_netcdf(sample_files.EXAMPLE_SONDE1)
    display = act.plotting.SkewTDisplay(ds)
    overwrite_data = {'Test': 1234.0}
    display.plot_enhanced_skewt(
        spd_name='u_wind',
        dir_name='v_wind',
        color_field='alt',
        component_range=85,
        uv_flag=True,
        overwrite_data=overwrite_data,
        add_data=overwrite_data,
    )
    ds.close()
    return display.fig


@pytest.mark.mpl_image_compare(tolerance=10)
def test_skewt_options():
    sonde_ds = act.io.arm.read_arm_netcdf(sample_files.EXAMPLE_SONDE1)
    skewt = SkewTDisplay(sonde_ds)
    skewt.plot_from_u_and_v(
        'u_wind',
        'v_wind',
        'pres',
        'tdry',
        'dp',
        plot_dry_adiabats=True,
        plot_moist_adiabats=True,
        plot_mixing_lines=True,
    )
    sonde_ds.close()
    return skewt.fig



"""
IMPROVE Data
-----------

This example shows how to get IMPROVE data for the
system located at ARM's Southern Great Plains site.

"""

import act
import matplotlib.pyplot as plt

# Pull the data using the site_id from IMPROVE
# https://views.cira.colostate.edu/adms/Pub/SiteSummary.aspx?dsidse=10001&siidse=244
# Data from the IMPROVE station next to ARM's BNF deployment
# can be found using site_id='21'
ds = act.discovery.get_improve_data(site_id='244', start_date='1/1/2023', end_date='12/31/2023')

# Remove all data that's set to the FillValue
ds = ds.where(ds['aluminum_fine'] != ds['aluminum_fine'].attrs['_FillValue'])

display = act.plotting.TimeSeriesDisplay(ds, figsize=(10, 6))
display.plot('aluminum_fine')

# Print out the known problems documented by IMPROVE
print(ds.attrs['site_problems'])

# Write out the data to netCDF and csv
ds.to_netcdf('./sgpimprove.20230101.nc')
ds.to_dataframe().to_csv('sgpimprove.20230101.csv')

plt.show()



"""
Airnow Data
-----------

This example shows the different ways to pull
air quality information from EPA's AirNow API for
a station near to SGP

"""

import os
import matplotlib.pyplot as plt
import act

# You need an account and token from https://docs.airnowapi.org/ first
token = os.getenv('AIRNOW_API')

if token is not None and len(token) > 0:
    # This first example will get the forcasted values for the date passed
    # at stations within 100 miles of the Zipcode. Can also use latlon instead as
    # results = act.discovery.get_airnow_forecast(token, '2022-05-01', distance=100,
    #                                             latlon=[41.958, -88.12])
    # If the username and token are not set, use the existing sample file
    results = act.discovery.get_airnow_forecast(token, '2022-05-01', zipcode=74630, distance=100)

    # The results show a dataset with air quality information from Oklahoma City
    # The data is not indexed by time and just a rudimentary xarray object from
    # converted from a pandas DataFrame.  Note that the AirNow API labels the data
    # returned as AQI.
    print(results)

    # This call gives the daily average for Ozone, PM2.5 and PM10
    results = act.discovery.get_airnow_obs(token, date='2022-05-01', zipcode=74630, distance=100)
    print(results)

    # This call will get all the station data for a time period within
    # the bounding box provided.  This will return the object with time
    # as a coordinate and can be used with ACT Plotting to plot after
    # squeezing the dimensions.  It can be a 2D time series
    lat_lon = '-98.172,35.879,-96.76,37.069'
    results = act.discovery.get_airnow_bounded_obs(
        token, '2022-05-01T00', '2022-05-01T12', lat_lon, 'OZONE,PM25', data_type='B'
    )
    # Reduce to 1D timeseries
    results = results.squeeze(dim='sites', drop=False)
    print(results)

    # Plot out data but note that Ozone was not return in the results
    display = act.plotting.TimeSeriesDisplay(results)
    display.plot('PM2.5', label='PM2.5')
    display.plot('AQI', label='AQI')
    plt.legend()
    plt.show()



# Place python module imports here, example:
import os
import matplotlib.pyplot as plt
import act

# Place arm username and token or example file if username and token
# aren't set, example:
username = os.getenv('ARM_USERNAME')
token = os.getenv('ARM_PASSWORD')

# Download and read file or files with the IO and discovery functions
# within ACT, example:
results = act.discovery.download_arm_data(
    username, token, 'sgpceilC1.b1', '2022-01-14', '2022-01-19'
)
ceil_ds = act.io.arm.read_arm_netcdf(results)

# Plot file using the ACT display submodule, example:
display = act.plotting.TimeSeriesDisplay(ceil_ds)
display.plot('backscatter')
plt.show()
